#include <iostream>

#include <algorithm>
#include <iterator>
#include <functional>

#include <set>
#include <list>

#include <ctime>

using namespace std;

class RandIntGenerator
{
private:
	int maxVal;
	int minVal;

public:
	RandIntGenerator (int maxVal, int minVal = 0) : maxVal(maxVal), minVal(minVal) { }
	int operator()() { return (rand() % (maxVal - minVal)) + minVal; }
};

class Timer
{
private:
	clock_t begin;

public:
	Timer() : begin(clock()) { }

	void start()
	{
		begin = clock();
	}

	double stop()
	{
		clock_t end = clock();
		return double(end - begin) / CLOCKS_PER_SEC;
	}
};

int main()
{
	srand (time(0));

	cout << "Random generalt ertekek: " << endl;
	int test[20];
	generate_n (test, 20, RandIntGenerator(10));
	copy (test, test + 20, ostream_iterator<int>(cout, " "));

	cout << endl << "Halmazba masolva: " << endl;
	set<int> testSet;
	copy(test, test + 20, inserter(testSet, testSet.begin()));
	copy(testSet.begin(), testSet.end(), ostream_iterator<int>(cout, " "));

    cout << endl << "Elemek duplazasa: " << endl;
    transform(testSet.begin(), testSet.end(), ostream_iterator<int>(cout, " "),
												bind1st(multiplies<int>(), 2));

	cout << endl << "Zsakba masolas: " << endl;
	multiset<int> testMultiSet;
	copy (test, test + 20, inserter(testMultiSet, testMultiSet.begin()));
	copy(testMultiSet.begin(), testMultiSet.end(), ostream_iterator<int>(cout, " "));

	cout << endl << endl << "Nagy adat teszt..." << endl;

	int bigTest[100000];

	Timer timer;

	timer.start();
	generate_n (bigTest, 100000, RandIntGenerator(1000000));
	cout << "Szamok generalasa: " << timer.stop() << " mp" << endl;

	list<int> bigList;
	timer.start();
	copy (bigTest, bigTest + 100000, inserter(bigList, bigList.begin()));
	cout << "Listaba masolas: " << timer.stop() << " mp" << endl;

	set<int> bigSet;
	timer.start();
	copy (bigTest, bigTest + 100000, inserter(bigSet, bigSet.begin()));
	cout << "Halmazba masolas: " << timer.stop() << " mp" << endl;

	int elementsToFind[10000];
	generate_n (elementsToFind, 10000, RandIntGenerator(1000000, 900000));

	timer.start();
	for (int i = 0; i < 10000; ++i)
		find (bigTest, bigTest + 100000, elementsToFind[i]);
	cout << "Tombben keres: " << timer.stop() << " mp" << endl;


	timer.start();
	for (int i = 0; i < 10000; ++i)
		find (bigList.begin(), bigList.end(), elementsToFind[i]);
	cout << "Listaban kereses: " << timer.stop() << " mp" << endl;


	timer.start();
	for (int i = 0; i < 10000; ++i)
		find (bigSet.begin(), bigSet.end(), elementsToFind[i]);
	cout << "Halmazban linearis kereses: " << timer.stop() << " mp" << endl;
	cout << "(Igy soha ne csinald!)" << endl;


	timer.start();
	for (int i = 0; i < 10000; ++i)
		bigSet.find(elementsToFind[i]);
	cout << "Halmazban logaritmikus kereses: " << timer.stop() << " mp" << endl;

    return 0;
}
